#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2022-06-28
# file  : inv_opengear_base
#
# opengear base inventory
#
# 2022-11-08: fixed crash in parse function on empty inv_data
# 2022-11-09: removed ' ()' from firmware version
# 2022-11-12: added support for CM81xx series (sysObjectID = .1.3.6.1.4.1.25049.1.101)
# 2024-05-10: added CM8132 to device list
#
# sample string_table
# [
#  [
#   ['4.1.1u2 594cb83d ()', 'N/A']
#  ],
#  [
#   ['1'], ['1'], ['1'], ['1'], ['1'], ['1'], ['1'], ['1'], ['1'], ['1'], ['2'], ['2'], ['2'], ['2'], ['2'], ['2']
#  ],
#  [
#   ['.1.3.6.1.4.1.25049.1.60']
#  ]
# ]
#

from typing import List

from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    InventoryResult,
    StringTable,
)
from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    Attributes,
    SNMPTree,
    register,
    startswith,
)

# from the OG-PRODUCTS-MIB, LAST-UPDATED "202005200000Z"
#
# ogCM4001 OBJECT IDENTIFIER ::= { ogProducts 1 }
# ogCM4002 OBJECT IDENTIFIER ::= { ogProducts 2 }
# ogCM4008 OBJECT IDENTIFIER ::= { ogProducts 3 }
# ogCM41xx OBJECT IDENTIFIER ::= { ogProducts 10 }
# ogCM71xx OBJECT IDENTIFIER ::= { ogProducts 11 }
# ogCM7196 OBJECT IDENTIFIER ::= { ogProducts 12 }
# ogSD4001 OBJECT IDENTIFIER ::= { ogProducts 20 }
# ogSD4002 OBJECT IDENTIFIER ::= { ogProducts 21 }
# ogSD4008 OBJECT IDENTIFIER ::= { ogProducts 22 }
# ogSD4001DW OBJECT IDENTIFIER ::= { ogProducts 23 }
# ogSD4002DX OBJECT IDENTIFIER ::= { ogProducts 24 }
# ogCD OBJECT IDENTIFIER ::= { ogProducts 30 }
# ogCMx86 OBJECT IDENTIFIER ::= { ogProducts 31 }
# ogCMS61xx OBJECT IDENTIFIER ::= { ogProducts 40 }
# ogLighthouse OBJECT IDENTIFIER ::= { ogProducts 41 }
# ogLighthouse5 OBJECT IDENTIFIER ::= { ogProducts 42 }
# ogIM4004 OBJECT IDENTIFIER ::= { ogProducts 50 }
# ogIM42xx OBJECT IDENTIFIER ::= { ogProducts 60 }
# ogIM72xx OBJECT IDENTIFIER ::= { ogProducts 61 }
# ogKCS61xx OBJECT IDENTIFIER ::= { ogProducts 70 }
# ogACM500x OBJECT IDENTIFIER ::= { ogProducts 80 }
# ogACM550x OBJECT IDENTIFIER ::= { ogProducts 81 }
# ogACM700x OBJECT IDENTIFIER ::= { ogProducts 90 }
# ogACM70045 OBJECT IDENTIFIER ::= { ogProducts 91 }
# ogOperationsManager OBJECT IDENTIFIER ::= { ogProducts 101 }
# ogOM12xx OBJECT IDENTIFIER ::= { ogProducts 110 }
# ogOM22xx OBJECT IDENTIFIER ::= { ogProducts 120 }
# ogCM81xx OBJECT IDENTIFIER ::= { ogProducts 130 }
# ogEM80xx OBJECT IDENTIFIER ::= { ogProducts 140 }
#

_opengear_models = {
    ('.1.3.6.1.4.1.25049.1.11', 18): 'CM7116',  # 16 serial + 2 usb lines
    ('.1.3.6.1.4.1.25049.1.11', 34): 'CM7132',  # 32 serial + 2 usb lines
    ('.1.3.6.1.4.1.25049.1.11', 50): 'CM7148',  # 48 serial + 2 usb lines
    ('.1.3.6.1.4.1.25049.1.60', 16): 'IM4216',  # 16 serial lines
    ('.1.3.6.1.4.1.25049.1.101', 34): 'CM8132',  # 32 serial + 2 usb lines
}


def parse_inv_opengear_base(string_table: List[StringTable]) -> List | None:
    try:
        inv_data, og_serial_port_cts, sys_object_id = string_table
    except ValueError:
        return

    try:
        firmware_version, serial_number = inv_data[0]
    except (IndexError, ValueError):
        return

    try:
        sys_object_id = sys_object_id[0][0]
    except (IndexError, ValueError):
        return

    if serial_number == 'N/A':
        serial_number = ''
    number_of_lines = len(og_serial_port_cts)
    model_name = _opengear_models.get((sys_object_id, number_of_lines), '')

    section = []

    inv_path = ['hardware', 'system']

    for key, value in [
        ('serial', serial_number),
        ('manufacturer', 'opengear'),
        ('model', model_name),
    ]:
        if not value == '':
            section.append((inv_path, key, value))

    inv_path = ['software', 'firmware']

    for key, value in [
        ('vendor', 'opengear'),
        ('version', firmware_version.rstrip(' ()')),
    ]:
        if not value == '':
            section.append((inv_path, key, value))

    return section


def inventory_opengear_base(section: List) -> InventoryResult:
    for invPath, key, value in section:
        yield Attributes(
            path=invPath,
            inventory_attributes={key: value})


register.snmp_section(
    name='inv_opengear_base',
    parse_function=parse_inv_opengear_base,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.25049.17.1',  # OG-STATUSv2-MIB
            oids=[
                '1',  # ogFirmwareVersion
                '2',  # ogSerialNumber
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.25049.17.2.1',  # OG-STATUSv2-MIB:orgSerialÜprtEntry
            oids=[
                '19',  # ogSerialPortCTS. ogSerialPortIndex don't exist, so we use CTS to get the number of lines
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.2.1.1',  # SNMPv2-MIB
            oids=[
                '2',  # sysObjectID
            ]
        ),
    ],
    detect=startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.25049'),
)


def parse_inv_opengear_base_om(string_table: StringTable) -> List | None:
    try:
        host_name, serial_number, firmware_version, vendor, model_name, number_of_lines = string_table[0]
    except IndexError:
        return

    section = []

    inv_path = ['hardware', 'system']

    for key, value in [
        ('serial', serial_number),
        ('manufacturer', vendor),
        ('model', model_name),
        ('host_name', host_name),
    ]:
        if not value == '':
            section.append((inv_path, key, value))

    inv_path = ['software', 'firmware']

    for key, value in [
        ('vendor', 'opengear'),
        ('version', firmware_version.rstrip(' ()')),
    ]:
        if not value == '':
            section.append((inv_path, key, value))

    return section


register.snmp_section(
    name='inv_opengear_base_om',
    parse_function=parse_inv_opengear_base_om,
    parsed_section_name='inv_opengear_base',
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.25049.10.19',  # OG-OMTELEM-MIB::ogOmTelem
        oids=[
            '1.1',  # ogOmSystemHostName
            '1.2',  # ogOmSystemSerialNumber
            '1.3',  # ogOmSystemFirmwareVersion
            '1.4',  # ogOmSystemVendor
            '1.5',  # ogOmSystemModel
            '2.1',  # ogOmSerialPortCount
        ]
    ),
    detect=startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.25049'),
)

register.inventory_plugin(
    name='inv_opengear_base',
    inventory_function=inventory_opengear_base,
)
